#include <stdio.h>
#include <time.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <dirent.h>
#include <unistd.h>
#include <qdos.h>



int getgroups (int n,gid_t g)
{
    return -1;
}
int access (char *f, int mode)
{
    struct stat st;
    int fd;
    
    if((fd = stat(f, &st)) == 0)
    {
	switch(fd)
	{
	case F_OK:
	    break;
	case R_OK:
	    fd = (st.st_mode & 0444) == 0;
	    break;
	case W_OK:
	    fd = (st.st_mode & 0222) == 0;
	    break;
	case X_OK:
	    fd = (st.st_mode & 0111) == 0;
	    break;
	default:
	    fd = -1;
	    break;
	}
    }
    return fd;
}

void Unix2MM (char *name, char **dot)
{
    char path[64];
    char *q, *r, *s;
    
    if(*name == '~')
    {
	r = name+1;
	getcwd(path, sizeof(path));
	q = path + strlen(path);
	if(*(q-1) != '_')
	{
	    *q++ = '_';
	    *q = 0;
	}	    
    }
    else
    {
	q = path;
	r = name;
    }
    
    if(*r == '/')
    {
	r++;
    }
    
    strcpy(q, r);

    while (*q)
    {
        if(*q == '/' || *q == '.')
	{
	    if(*q == '.' && dot)
	    {
		*dot = name + (q - path);
	    }
	    *q = '_';
	}
	
	q++;
    }
    strcpy(name, path);
}

GuessAltName(char *name, char *dot)
{
    if(dot)
    {
	*dot = '.';
    }
    else
    {
	if((dot = strrchr(name, '_')))
	{
	    *dot = '.';
	}
    }
}

void MM2Unix (char *name)
{
    struct stat st;
    int sts;
    char *p, *r, *s;
    char path[64];
    short pabs = 0;

    strcpy(path, name);
    
    if(isdirdev(name) && *(name+4) == '_')
    {
	p = name+5;
	*(path+4) = '/';
	pabs = 1;
    }
    else
    {
	p = name;
    }

    if(s = strrchr(p, '_'))
    {
	*s = 0;
	sts = stat(name, &st);
	if(sts ==0 && (st.st_mode & S_IFDIR))
	{
	    *(path+(s-name)) = '/';
	}
	else
	{
	    *(path+(s-name)) = '.';
	}
    }

    for(r = p; *r; r++)
    {
        if(*r == '_')
	{
	    *r = 0;
	    sts = stat(name, &st);
	    if(sts ==0 && (st.st_mode & S_IFDIR))
	    {
		*(path+(r-name)) = '/';
	    }
	    else
	    {
		*(path+(r-name)) = '_';
	    }
	    *r = '_';
	}
    }
    p = name;
    if(pabs)
    {
	*p++ = '/';
    }
    strcpy(p, path);
}

int ql_access(char *name, int mode)
{
    int res;
    char *dot = NULL;
    
    Unix2MM(name, &dot);
    res = access(name, mode);
    if(res != 0)
    {
	GuessAltName(name, dot);
	res = access(name, mode);
    }
    
#ifdef QLTEST
    fprintf(stderr,"ACCESS: %s = %d %p\n", name, res, dot);
#endif
    MM2Unix(name);
    return res;
}

char *ql_getcwd(char *buf, size_t sz)
{
    char *p,*q,*r,*s;
    struct stat st;
    int sts;

    getcwd(buf, sz);
    q = malloc(64);
    *q = '/';
    s = strcpy(q+1, buf);

    r = strchr(buf,'_') + 1;

    for(; *r; r++)
    {
        if(*r == '_')
	{
	    *r = 0;
	    sts = stat(buf, &st);
	    if(sts ==0 && (st.st_mode & S_IFDIR))
	    {
		*(s+(r-buf)) = '/';
	    }
	    *r = '_';
	}
    }
    
    s = strcpy(buf, q);
    free(q);
    q = s + strlen(s) -1;
    if(*q == '/') *q = 0;
    return s;
}

int ql_stat (char *name, struct stat *s)
{
    int res;
    char *dot = NULL;
    
    Unix2MM(name, &dot);
    res = stat (name, s);
    if(res != 0)
    {
	GuessAltName(name, dot);
	res = stat (name, s);
    }

#ifdef QLTEST
    fprintf(stderr,"STAT: %s = %d", name, res);
#endif
    MM2Unix(name);
    
    return res;
}

FILE *ql_fopen(char *name, char *mode)
{
    FILE *fp;
    char *dot = NULL;
    
    Unix2MM(name, &dot);
    fp = fopen(name, mode);
#ifdef QLTEST
    fprintf(stderr,"FOPEN: [%s] %p (%s)\n", name, fp, mode);
#endif

    if(fp == NULL && (*mode == 'a' || *mode == 'r'))
    {
	GuessAltName(name, dot);
	fp = fopen(name, mode);
    }
    
    MM2Unix(name);
    return fp;
}
int ql_open(char *name, int mode,...)
{
    int fd;
    char *dot = NULL;
    
    Unix2MM(name, &dot);
#ifdef QLTEST
    fprintf(stderr,"OPEN: %s\n", name);
#endif
    fd = open(name, mode);
    if(fd < 0)
    {
	GuessAltName(name, dot);
	fd = open(name, mode);
    }
    MM2Unix(name);
    return fd;
}

ql_rename(char *src, char *dest)
{
    int r = 0;
    char *dot = NULL;
    
    Unix2MM(src, &dot);
    r = access(src, F_OK);
    if(r != 0)
    {
	GuessAltName(src, dot);
	r = access(src, F_OK);
    }
    if(r == 0)
    {
	Unix2MM(dest, NULL);
	r = rename(src, dest);
	MM2Unix(src);
	MM2Unix(dest);
    }
    return r;
}

ql_unlink(char *src)
{
    int r;
    char *dot = NULL;

    Unix2MM(src, &dot);

    r = access(src, F_OK);

    if(r != 0)
    {
	GuessAltName(src, dot);
	r = access(src, F_OK);
    }
    
    if(r == 0)
    {
	r = unlink(src);
	MM2Unix(src);
    }
    return r;
}


